#!/bin/bash
### DavinciRecode v0.10
### Visit https://forum.blackmagicdesign.com/viewtopic.php?f=21&t=136365 for updates and discussion
# Variable frame rate videos will be reencoded to this FPS value
framerate=24
# If you prefer a different dir, don't forget the closing slash
# Comment to set the export path to current dir
folderprefix='exportedvideo/'
# ffmpeg location
#ffmpeg='/snap/bin/ffmpeg'
ffmpeg='ffmpeg'
# For DR studio users, set to false #display case
deinterlace=true
# Select target audio codec here. pcm_s24le is good for editing
audiocodec=pcm_s24le
# Select target video codec here. Use prores_ks here for editing
videocodec=h264
# Container
container=mkv
# Add something just before the end of ffmpeg command. ie '-profile:v 3' switch for prores_ks codec
#append='-crf 17' #for h264
#
whatdoweneed () {
	# 0 - do nothing
	# 1 - only repackage to $container
	# 2 - do only CFR
	# 3 - do audio + repackage to $container
	# 4 - CFR + audio + repackage
	# 5 - Deinterlace + CFR + audio
	# 6 - Deinterlace + CFR
	# 7 - audio + deinterlace
	# 8 - deinterlace
	if [ `mediainfo "$1" | grep 'Frame rate mode' | grep 'Variable' | wc -l` -gt "0" ]; then
	# VFR
		if [ `mediainfo "$1" | grep -i 'PCM\|ac3' | wc -l` -eq "0" ]; then
		# NOT PCM or ac3, so convert audio
			if [ `mediainfo "$1" | grep 'Interlaced' | wc -l` -gt "0" ]; then
			#Interlaced
				if [[ "$deinterlace" = "true" ]]; then
					return 5
				else
					return 4
				fi
			else
				return 4
			fi
		else
		# PCM or ac3, don't convert audio
			if [ `mediainfo "$1" | grep 'Interlaced' | wc -l` -gt "0" ]; then
			# interlaced
				if [[ "$deinterlace" = "true" ]]; then
					return 6
				else
					return 2
				fi
			else
				return 2
			fi
		fi
	else
	# CFR
		if [ `mediainfo "$1" | grep ': PCM\|ac3' | wc -l` -eq "0" ]; then
		# NOT PCM or ac3, so convert audio
			if [ `mediainfo "$1" | grep 'Interlaced' | wc -l` -gt "0" ]; then
			# interlaced
				if [[ "$deinterlace" = "true" ]]; then
					return 7
				else
					return 3
				fi
			else
				# is it an ok video codec?
				if [ `mediainfo "$1" | grep 'H.263\|VP8\|WMV3\|IV50\|MJPG' | wc -l` -gt "0" ]; then
					return 4
				else
					return 3
				fi
			fi
		else
			if [ `mediainfo "$1" | grep 'Interlaced' | wc -l` -gt "0" ]; then
			# interlaced
				if [[ "$deinterlace" = "true" ]]; then
					return 8
				else
					return 0
				fi
			else
				#if [ `mediainfo "$1" | grep 'Format' | grep 'AVI' | wc -l` -gt "0" ] || [ `mediainfo "$1" | grep 'Format' | grep 'Flash Video' | wc -l` -gt "0" ]; then
				# wrong container
				if [ `mediainfo "$1" | grep 'H.263\|VP8\|WMV3\|IV50\|MJPG' | wc -l` -gt "0" ]; then
					#echo "case BUG"
					return 4
				else
					return 0
				fi
			fi
		fi
	fi
}
if [ -n "$folderprefix" ]; then
	mkdir "$folderprefix"
fi
for filename in *.*;do
	finalname=`echo "$filename" | rev | cut -d'.' -f 2- | rev`
	whatdoweneed "$filename"
	resultset="$?"
	case "$resultset" in
		0)
			echo "case 0 - $filename : Skipping (copy to dest)"
			cp -a "$filename" "$folderprefix""copy_$filename"
		;;
		1)
			echo "case 1 - $filename : Converting to $container"
			#ffmpeg -hide_banner -loglevel error -i "$filename" -f mov "$finalname""_container.mov" #broken pcm. Fix me!
			$ffmpeg -hide_banner -loglevel error -i "$filename" -acodec copy -vcodec copy "$folderprefix""$finalname""_container.$container"
			touch -r "$filename" "$folderprefix""$finalname""_container.$container"
		;;
		2)
			echo "case 2 - $filename : Converting to constant framerate $videocodec and $container "
			$ffmpeg -hide_banner -loglevel error -i "$filename" -acodec copy -r $framerate -vcodec $videocodec $append "$folderprefix""$finalname""_cfr.$container"
			touch -r "$filename" "$folderprefix""$finalname""_cfr.$container"
		;;
		3)
			echo "case 3 - $filename : Converting audio only $container"
			$ffmpeg -hide_banner -loglevel error -i "$filename" -vcodec copy -acodec $audiocodec $append "$folderprefix""$finalname""_audio.$container"
			touch -r "$filename" "$folderprefix""$finalname""_audio.$container"
		;;
		4)
			echo "case 4 - $filename : Converting audio and constant framerate $videocodec $container"
			$ffmpeg -hide_banner -loglevel error -i "$filename" -acodec $audiocodec -vcodec $videocodec -r $framerate -max_muxing_queue_size 1024 $append "$folderprefix""$finalname""_cfr_audio.$container"
			touch -r "$filename" "$folderprefix""$finalname""_cfr_audio.$container"
		;;
		5)
			echo "case 5 - $filename : Converting audio + constant framerate + deinterlace $videocodec $container"
			$ffmpeg -hide_banner -loglevel error -i "$filename" -acodec $audiocodec -vcodec $videocodec -r $framerate -vf yadif $append "$folderprefix""$finalname""_cfr_audio_deint.$container"
			touch -r "$filename" "$folderprefix""$finalname""_cfr_audio_deint.$container"
		;;
		6)
			echo "case 6 - $filename : Converting to constant framerate and deinterlace $videocodec $container"
			$ffmpeg -hide_banner -loglevel error -i "$filename" -vcodec $videocodec -r $framerate -vf yadif $append "$folderprefix""$finalname""_cfr_deint.$container"
			touch -r "$filename" "$folderprefix""$finalname""_cfr_deint.$container"
		;;
		7)
			echo "case 7 - $filename : Converting audio and deinterlacing $videocodec $container"
			$ffmpeg -hide_banner -loglevel error -i "$filename" -acodec $audiocodec -vcodec $videocodec -vf yadif $append "$folderprefix""$finalname""_audio_deint.$container"
			touch -r "$filename" "$folderprefix""$finalname""_audio_deint.$container"
		;;
		8)
			echo "case 8 - $filename : Deinterlace $videocodec $container"
			$ffmpeg -hide_banner -loglevel error -i "$filename" -acodec copy -vcodec $videocodec -vf yadif $append "$folderprefix""$finalname""_deint.$container"
			touch -r "$filename" "$folderprefix""$finalname""_deint.$container"
		;;
		*)
			echo "$filename ---BUG---"
		;;
	esac
done
